@@ -42,14 +42,14 @@ test -x /usr/bin/lcov && make cov
 rm -f *.tar.gz
 make dist
 
-if [ -f /usr/bin/rpmbuild ]; then
-  if [ -n "$AUTOBUILD_COUNTER" ]; then
-    EXTRA_RELEASE=".auto$AUTOBUILD_COUNTER"
-  else
-    NOW=`date +"%s"`
-    EXTRA_RELEASE=".$USER$NOW"
-  fi
+if [ -n "$AUTOBUILD_COUNTER" ]; then
+  EXTRA_RELEASE=".auto$AUTOBUILD_COUNTER"
+else
+  NOW=`date +"%s"`
+  EXTRA_RELEASE=".$USER$NOW"
+fi
 
+if [ -f /usr/bin/rpmbuild ]; then
   rpmbuild --nodeps \
      --define "extra_release $EXTRA_RELEASE" \
      --define "_sourcedir `pwd`" \
@@ -65,6 +65,7 @@ if [ -x /usr/bin/i686-pc-mingw32-gcc ]; then
     --build=$(uname -m)-pc-linux \
     --host=i686-pc-mingw32 \
     --prefix="$AUTOBUILD_INSTALL_ROOT/i686-pc-mingw32/sys-root/mingw" \
+    --enable-compile-warnings=error \
     --without-sasl \
     --without-avahi \
     --without-polkit \
@@ -72,6 +73,7 @@ if [ -x /usr/bin/i686-pc-mingw32-gcc ]; then
     --without-xen \
     --without-qemu \
     --without-lxc \
+    --without-uml \
     --without-openvz \
     --without-libvirtd
 
@@ -81,8 +83,10 @@ if [ -x /usr/bin/i686-pc-mingw32-gcc ]; then
   #set -o pipefail
   #make check 2>&1 | tee "$RESULTS"
 
-  rpmbuild --nodeps \
-     --define "extra_release $EXTRA_RELEASE" \
-     --define "_sourcedir `pwd`" \
-     -ba --clean mingw-libvirt.spec
+  if [ -f /usr/bin/rpmbuild ]; then
+    rpmbuild --nodeps \
+       --define "extra_release $EXTRA_RELEASE" \
+       --define "_sourcedir `pwd`" \
+       -ba --clean mingw32-libvirt.spec
+  fi
 fi
