@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007 Red Hat, Inc.
+ * Copyright (C) 2007, 2009 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,7 +21,7 @@
 
 #include <config.h>
 
-#if defined(WITH_QEMU) || defined(WITH_LXC)
+#if defined(WITH_BRIDGE)
 
 #include "bridge.h"
 
@@ -47,8 +47,9 @@
 #include "internal.h"
 #include "memory.h"
 #include "util.h"
+#include "logging.h"
 
-#define MAX_BRIDGE_ID 256
+#define MAX_TAP_ID 256
 
 #define JIFFIES_TO_MS(j) (((j)*1000)/HZ)
 #define MS_TO_JIFFIES(ms) (((ms)*HZ)/1000)
@@ -126,32 +127,13 @@ brShutdown(brControl *ctl)
 #ifdef SIOCBRADDBR
 int
 brAddBridge(brControl *ctl,
-            char **name)
+            const char *name)
 {
     if (!ctl || !ctl->fd || !name)
         return EINVAL;
 
-    if (*name) {
-        if (ioctl(ctl->fd, SIOCBRADDBR, *name) == 0)
-            return 0;
-    } else {
-        int id = 0;
-        do {
-            char try[50];
-
-            snprintf(try, sizeof(try), "virbr%d", id);
-
-            if (ioctl(ctl->fd, SIOCBRADDBR, try) == 0) {
-                if (!(*name = strdup(try))) {
-                    ioctl(ctl->fd, SIOCBRDELBR, name);
-                    return ENOMEM;
-                }
-                return 0;
-            }
-
-            id++;
-        } while (id < MAX_BRIDGE_ID);
-    }
+    if (ioctl(ctl->fd, SIOCBRADDBR, name) == 0)
+        return 0;
 
     return errno;
 }
@@ -163,6 +145,43 @@ int brAddBridge (brControl *ctl ATTRIBUTE_UNUSED,
 }
 #endif
 
+#ifdef SIOCBRDELBR
+int
+brHasBridge(brControl *ctl,
+            const char *name)
+{
+    struct ifreq ifr;
+    int len;
+
+    if (!ctl || !name) {
+        errno = EINVAL;
+        return -1;
+    }
+
+    if ((len = strlen(name)) >= BR_IFNAME_MAXLEN) {
+        errno = EINVAL;
+        return -1;
+    }
+
+    memset(&ifr, 0, sizeof(struct ifreq));
+
+    strncpy(ifr.ifr_name, name, len);
+    ifr.ifr_name[len] = '\0';
+
+    if (ioctl(ctl->fd, SIOCGIFFLAGS, &ifr))
+        return -1;
+
+    return 0;
+}
+#else
+int
+brHasBridge(brControl *ctl,
+            const char *name)
+{
+    return EINVAL;
+}
+#endif
+
 /**
  * brDeleteBridge:
  * @ctl: bridge control pointer
@@ -275,11 +294,159 @@ brDeleteInterface(brControl *ctl ATTRIBUTE_UNUSED,
 }
 #endif
 
+/**
+ * ifGetMtu
+ * @ctl: bridge control pointer
+ * @ifname: interface name get MTU for
+ *
+ * This function gets the @mtu value set for a given interface @ifname.
+ *
+ * Returns the MTU value in case of success.
+ * On error, returns -1 and sets errno accordingly
+ */
+static int ifGetMtu(brControl *ctl, const char *ifname)
+{
+    struct ifreq ifr;
+    int len;
+
+    if (!ctl || !ifname) {
+        errno = EINVAL;
+        return -1;
+    }
+
+    if ((len = strlen(ifname)) >=  BR_IFNAME_MAXLEN) {
+        errno = EINVAL;
+        return -1;
+    }
+
+    memset(&ifr, 0, sizeof(struct ifreq));
+
+    strncpy(ifr.ifr_name, ifname, len);
+    ifr.ifr_name[len] = '\0';
+
+    if (ioctl(ctl->fd, SIOCGIFMTU, &ifr))
+        return -1;
+
+    return ifr.ifr_mtu;
+
+}
+
+/**
+ * ifSetMtu:
+ * @ctl: bridge control pointer
+ * @ifname: interface name to set MTU for
+ * @mtu: MTU value
+ *
+ * This function sets the @mtu for a given interface @ifname.  Typically
+ * used on a tap device to set up for Jumbo Frames.
+ *
+ * Returns 0 in case of success or an errno code in case of failure.
+ */
+static int ifSetMtu(brControl *ctl, const char *ifname, int mtu)
+{
+    struct ifreq ifr;
+    int len;
+
+    if (!ctl || !ifname)
+        return EINVAL;
+
+    if ((len = strlen(ifname)) >=  BR_IFNAME_MAXLEN)
+        return EINVAL;
+
+    memset(&ifr, 0, sizeof(struct ifreq));
+
+    strncpy(ifr.ifr_name, ifname, len);
+    ifr.ifr_name[len] = '\0';
+    ifr.ifr_mtu = mtu;
+
+    return ioctl(ctl->fd, SIOCSIFMTU, &ifr) == 0 ? 0 : errno;
+}
+
+/**
+ * brSetInterfaceMtu
+ * @ctl: bridge control pointer
+ * @bridge: name of the bridge interface
+ * @ifname: name of the interface whose MTU we want to set
+ *
+ * Sets the interface mtu to the same MTU of the bridge
+ *
+ * Returns 0 in case of success or an errno code in case of failure.
+ */
+static int brSetInterfaceMtu(brControl *ctl,
+                             const char *bridge,
+                             const char *ifname)
+{
+    int mtu = ifGetMtu(ctl, bridge);
+
+    if (mtu < 0)
+        return errno;
+
+    return ifSetMtu(ctl, ifname, mtu);
+}
+
+/**
+ * brProbeVnetHdr:
+ * @tapfd: a tun/tap file descriptor
+ *
+ * Check whether it is safe to enable the IFF_VNET_HDR flag on the
+ * tap interface.
+ *
+ * Setting IFF_VNET_HDR enables QEMU's virtio_net driver to allow
+ * guests to pass larger (GSO) packets, with partial checksums, to
+ * the host. This greatly increases the achievable throughput.
+ *
+ * It is only useful to enable this when we're setting up a virtio
+ * interface. And it is only *safe* to enable it when we know for
+ * sure that a) qemu has support for IFF_VNET_HDR and b) the running
+ * kernel implements the TUNGETIFF ioctl(), which qemu needs to query
+ * the supplied tapfd.
+ *
+ * Returns 0 in case of success or an errno code in case of failure.
+ */
+static int
+brProbeVnetHdr(int tapfd)
+{
+#if defined(IFF_VNET_HDR) && defined(TUNGETFEATURES) && defined(TUNGETIFF)
+    unsigned int features;
+    struct ifreq dummy;
+
+    if (ioctl(tapfd, TUNGETFEATURES, &features) != 0) {
+        VIR_INFO0(_("Not enabling IFF_VNET_HDR; "
+                    "TUNGETFEATURES ioctl() not implemented"));
+        return 0;
+    }
+
+    if (!(features & IFF_VNET_HDR)) {
+        VIR_INFO0(_("Not enabling IFF_VNET_HDR; "
+                    "TUNGETFEATURES ioctl() reports no IFF_VNET_HDR"));
+        return 0;
+    }
+
+    /* The kernel will always return -1 at this point.
+     * If TUNGETIFF is not implemented then errno == EBADFD.
+     */
+    if (ioctl(tapfd, TUNGETIFF, &dummy) != -1 || errno != EBADFD) {
+        VIR_INFO0(_("Not enabling IFF_VNET_HDR; "
+                    "TUNGETIFF ioctl() not implemented"));
+        return 0;
+    }
+
+    VIR_INFO0(_("Enabling IFF_VNET_HDR"));
+
+    return 1;
+#else
+    (void) tapfd;
+    VIR_INFO0(_("Not enabling IFF_VNET_HDR; disabled at build time"));
+    return 0;
+#endif
+}
+
 /**
  * brAddTap:
  * @ctl: bridge control pointer
  * @bridge: the bridge name
  * @ifname: the interface name (or name template)
+ * @vnet_hdr: whether to try enabling IFF_VNET_HDR
  * @tapfd: file descriptor return value for the new tap device
  *
  * This function creates a new tap device on a bridge. @ifname can be either
@@ -293,6 +460,7 @@ int
 brAddTap(brControl *ctl,
          const char *bridge,
          char **ifname,
+         int vnet_hdr,
          int *tapfd)
 {
     int id, subst, fd;
@@ -308,6 +476,9 @@ brAddTap(brControl *ctl,
     if ((fd = open("/dev/net/tun", O_RDWR)) < 0)
       return errno;
 
+    if (vnet_hdr)
+        vnet_hdr = brProbeVnetHdr(fd);
+
     do {
         struct ifreq try;
         int len;
@@ -316,6 +487,11 @@ brAddTap(brControl *ctl,
 
         try.ifr_flags = IFF_TAP|IFF_NO_PI;
 
+#ifdef IFF_VNET_HDR
+        if (vnet_hdr)
+            try.ifr_flags |= IFF_VNET_HDR;
+#endif
+
         if (subst) {
             len = snprintf(try.ifr_name, BR_IFNAME_MAXLEN, *ifname, id);
             if (len >= BR_IFNAME_MAXLEN) {
@@ -334,21 +510,25 @@ brAddTap(brControl *ctl,
         }
 
         if (ioctl(fd, TUNSETIFF, &try) == 0) {
+            /* We need to set the interface MTU before adding it
+             * to the bridge, because the bridge will have its
+             * MTU adjusted automatically when we add the new interface.
+             */
+            if ((errno = brSetInterfaceMtu(ctl, bridge, try.ifr_name)))
+                goto error;
             if ((errno = brAddInterface(ctl, bridge, try.ifr_name)))
                 goto error;
             if ((errno = brSetInterfaceUp(ctl, try.ifr_name, 1)))
                 goto error;
             VIR_FREE(*ifname);
-            if (!(*ifname = strdup(try.ifr_name))) {
-                errno = ENOMEM;
+            if (!(*ifname = strdup(try.ifr_name)))
                 goto error;
-            }
             *tapfd = fd;
             return 0;
         }
 
         id++;
-    } while (subst && id <= MAX_BRIDGE_ID);
+    } while (subst && id <= MAX_TAP_ID);
 
  error:
     close(fd);
@@ -444,6 +624,10 @@ brSetInetAddr(brControl *ctl,
               int cmd,
               const char *addr)
 {
+    union {
+        struct sockaddr sa;
+        struct sockaddr_in sa_in;
+    } s;
     struct ifreq ifr;
     struct in_addr inaddr;
     int len, ret;
@@ -464,8 +648,10 @@ brSetInetAddr(brControl *ctl,
     else if (ret == 0)
         return EINVAL;
 
-    ((struct sockaddr_in *)&ifr.ifr_data)->sin_family = AF_INET;
-    ((struct sockaddr_in *)&ifr.ifr_data)->sin_addr   = inaddr;
+    s.sa_in.sin_family = AF_INET;
+    s.sa_in.sin_addr   = inaddr;
+
+    ifr.ifr_addr = s.sa;
 
     if (ioctl(ctl->fd, cmd, &ifr) < 0)
         return errno;
@@ -654,4 +840,4 @@ brSetEnableSTP(brControl *ctl ATTRIBUTE_UNUSED,
     return 0;
 }
 
-#endif /* WITH_QEMU || WITH_LXC */
+#endif /* WITH_BRIDGE */
