@@ -29,6 +29,7 @@
 #include "internal.h"
 #include "domain_conf.h"
 #include "capabilities.h"
+#include "threads.h"
 
 #define LXC_CONFIG_DIR SYSCONF_DIR "/libvirt/lxc"
 #define LXC_STATE_DIR LOCAL_STATE_DIR "/run/libvirt/lxc"
@@ -36,9 +37,11 @@
 
 typedef struct __lxc_driver lxc_driver_t;
 struct __lxc_driver {
+    virMutex lock;
+
     virCapsPtr caps;
 
-    virDomainObjPtr domains;
+    virDomainObjList domains;
     char *configDir;
     char *autostartDir;
     char *stateDir;
@@ -49,10 +52,9 @@ struct __lxc_driver {
 int lxcLoadDriverConfig(lxc_driver_t *driver);
 virCapsPtr lxcCapsInit(void);
 
-void lxcError(virConnectPtr conn,
-              virDomainPtr dom,
-              int code, const char *fmt, ...)
-    ATTRIBUTE_FORMAT(printf,4,5);
+#define lxcError(conn, dom, code, fmt...)                                    \
+        virReportErrorHelper(conn, VIR_FROM_LXC, code, __FILE__,           \
+                               __FUNCTION__, __LINE__, fmt)
 
 #endif /* LXC_CONF_H */
 
