@@ -30,10 +30,11 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <fcntl.h>
+#include <string.h>
 #include <dirent.h>
 
-#include "internal.h"
-
+#include "virterror_internal.h"
+#include "datatypes.h"
 #include "network_conf.h"
 #include "memory.h"
 #include "xml.h"
@@ -42,54 +43,44 @@
 #include "buf.h"
 #include "c-ctype.h"
 
+#define MAX_BRIDGE_ID 256
+#define VIR_FROM_THIS VIR_FROM_NETWORK
+
 VIR_ENUM_DECL(virNetworkForward)
 
 VIR_ENUM_IMPL(virNetworkForward,
               VIR_NETWORK_FORWARD_LAST,
               "none", "nat", "route" )
 
-static void virNetworkReportError(virConnectPtr conn,
-                                  int code, const char *fmt, ...)
-{
-    va_list args;
-    char errorMessage[1024];
-    const char *virerr;
+#define virNetworkReportError(conn, code, fmt...)                            \
+        virReportErrorHelper(conn, VIR_FROM_NETWORK, code, __FILE__,       \
+                               __FUNCTION__, __LINE__, fmt)
 
-    if (fmt) {
-        va_start(args, fmt);
-        vsnprintf(errorMessage, sizeof(errorMessage)-1, fmt, args);
-        va_end(args);
-    } else {
-        errorMessage[0] = '\0';
-    }
-
-    virerr = __virErrorMsg(code, (errorMessage[0] ? errorMessage : NULL));
-    __virRaiseError(conn, NULL, NULL, VIR_FROM_NETWORK, code, VIR_ERR_ERROR,
-                    virerr, errorMessage, NULL, -1, -1, virerr, errorMessage);
-}
-
-
-virNetworkObjPtr virNetworkFindByUUID(const virNetworkObjPtr nets,
+virNetworkObjPtr virNetworkFindByUUID(const virNetworkObjListPtr nets,
                                       const unsigned char *uuid)
 {
-    virNetworkObjPtr net = nets;
-    while (net) {
-        if (!memcmp(net->def->uuid, uuid, VIR_UUID_BUFLEN))
-            return net;
-        net = net->next;
+    unsigned int i;
+
+    for (i = 0 ; i < nets->count ; i++) {
+        virNetworkObjLock(nets->objs[i]);
+        if (!memcmp(nets->objs[i]->def->uuid, uuid, VIR_UUID_BUFLEN))
+            return nets->objs[i];
+        virNetworkObjUnlock(nets->objs[i]);
     }
 
     return NULL;
 }
 
-virNetworkObjPtr virNetworkFindByName(const virNetworkObjPtr nets,
+virNetworkObjPtr virNetworkFindByName(const virNetworkObjListPtr nets,
                                       const char *name)
 {
-    virNetworkObjPtr net = nets;
-    while (net) {
-        if (STREQ(net->def->name, name))
-            return net;
-        net = net->next;
+    unsigned int i;
+
+    for (i = 0 ; i < nets->count ; i++) {
+        virNetworkObjLock(nets->objs[i]);
+        if (STREQ(nets->objs[i]->def->name, name))
+            return nets->objs[i];
+        virNetworkObjUnlock(nets->objs[i]);
     }
 
     return NULL;
@@ -135,19 +126,29 @@ void virNetworkObjFree(virNetworkObjPtr net)
     virNetworkDefFree(net->def);
     virNetworkDefFree(net->newDef);
 
-    VIR_FREE(net->configFile);
-    VIR_FREE(net->autostartLink);
+    virMutexDestroy(&net->lock);
 
     VIR_FREE(net);
 }
 
+void virNetworkObjListFree(virNetworkObjListPtr nets)
+{
+    unsigned int i;
+
+    for (i = 0 ; i < nets->count ; i++)
+        virNetworkObjFree(nets->objs[i]);
+
+    VIR_FREE(nets->objs);
+    nets->count = 0;
+}
+
 virNetworkObjPtr virNetworkAssignDef(virConnectPtr conn,
-                                     virNetworkObjPtr *nets,
+                                     virNetworkObjListPtr nets,
                                      const virNetworkDefPtr def)
 {
     virNetworkObjPtr network;
 
-    if ((network = virNetworkFindByName(*nets, def->name))) {
+    if ((network = virNetworkFindByName(nets, def->name))) {
         if (!virNetworkIsActive(network)) {
             virNetworkDefFree(network->def);
             network->def = def;
@@ -161,39 +162,56 @@ virNetworkObjPtr virNetworkAssignDef(virConnectPtr conn,
     }
 
     if (VIR_ALLOC(network) < 0) {
-        virNetworkReportError(conn, VIR_ERR_NO_MEMORY, NULL);
+        virReportOOMError(conn);
+        return NULL;
+    }
+    if (virMutexInit(&network->lock) < 0) {
+        virNetworkReportError(conn, VIR_ERR_INTERNAL_ERROR,
+                              "%s", _("cannot initialize mutex"));
+        VIR_FREE(network);
+        return NULL;
+    }
+    virNetworkObjLock(network);
+    network->def = def;
+
+    if (VIR_REALLOC_N(nets->objs, nets->count + 1) < 0) {
+        virReportOOMError(conn);
+        VIR_FREE(network);
         return NULL;
     }
 
-    network->def = def;
-    network->next = *nets;
-
-    *nets = network;
+    nets->objs[nets->count] = network;
+    nets->count++;
 
     return network;
 
 }
 
-void virNetworkRemoveInactive(virNetworkObjPtr *nets,
+void virNetworkRemoveInactive(virNetworkObjListPtr nets,
                               const virNetworkObjPtr net)
 {
-    virNetworkObjPtr prev = NULL;
-    virNetworkObjPtr curr = *nets;
+    unsigned int i;
 
-    while (curr &&
-           curr != net) {
-        prev = curr;
-        curr = curr->next;
+    virNetworkObjUnlock(net);
+    for (i = 0 ; i < nets->count ; i++) {
+        virNetworkObjLock(nets->objs[i]);
+        if (nets->objs[i] == net) {
+            virNetworkObjUnlock(nets->objs[i]);
+            virNetworkObjFree(nets->objs[i]);
+
+            if (i < (nets->count - 1))
+                memmove(nets->objs + i, nets->objs + i + 1,
+                        sizeof(*(nets->objs)) * (nets->count - (i + 1)));
+
+            if (VIR_REALLOC_N(nets->objs, nets->count - 1) < 0) {
+                ; /* Failure to reduce memory allocation isn't fatal */
+            }
+            nets->count--;
+
+            break;
+        }
+        virNetworkObjUnlock(nets->objs[i]);
     }
-
-    if (curr) {
-        if (prev)
-            prev->next = curr->next;
-        else
-            *nets = curr->next;
-    }
-
-    virNetworkObjFree(net);
 }
 
 
@@ -223,7 +241,7 @@ virNetworkDHCPRangeDefParseXML(virConnectPtr conn,
             if (VIR_REALLOC_N(def->ranges, def->nranges + 1) < 0) {
                 xmlFree(start);
                 xmlFree(end);
-                virNetworkReportError(conn, VIR_ERR_NO_MEMORY, NULL);
+                virReportOOMError(conn);
                 return -1;
             }
             def->ranges[def->nranges].start = (char *)start;
@@ -274,7 +292,7 @@ virNetworkDHCPRangeDefParseXML(virConnectPtr conn,
                 VIR_FREE(ip);
                 VIR_FREE(mac);
                 VIR_FREE(name);
-                virNetworkReportError(conn, VIR_ERR_NO_MEMORY, NULL);
+                virReportOOMError(conn);
                 return -1;
             }
             def->hosts[def->nhosts].mac = (char *)mac;
@@ -297,7 +315,7 @@ virNetworkDefParseXML(virConnectPtr conn,
     char *tmp;
 
     if (VIR_ALLOC(def) < 0) {
-        virNetworkReportError(conn, VIR_ERR_NO_MEMORY, NULL);
+        virReportOOMError(conn);
         return NULL;
     }
 
@@ -314,7 +332,7 @@ virNetworkDefParseXML(virConnectPtr conn,
         int err;
         if ((err = virUUIDGenerate(def->uuid))) {
             virNetworkReportError(conn, VIR_ERR_INTERNAL_ERROR,
-                             _("Failed to generate UUID: %s"), strerror(err));
+                                  "%s", _("Failed to generate UUID"));
             goto error;
         }
     } else {
@@ -364,8 +382,8 @@ virNetworkDefParseXML(virConnectPtr conn,
         inaddress.s_addr &= innetmask.s_addr;
         netaddr = inet_ntoa(inaddress);
 
-        if (asprintf(&def->network, "%s/%s", netaddr, def->netmask) < 0) {
-            virNetworkReportError(conn, VIR_ERR_NO_MEMORY, NULL);
+        if (virAsprintf(&def->network, "%s/%s", netaddr, def->netmask) < 0) {
+            virReportOOMError(conn);
             goto error;
         }
 
@@ -453,7 +471,7 @@ virNetworkDefPtr virNetworkDefParseString(virConnectPtr conn,
     if (!xml) {
         if (conn && conn->err.code == VIR_ERR_NONE)
               virNetworkReportError(conn, VIR_ERR_XML_ERROR,
-                                    _("failed to parse xml document"));
+                                    "%s", _("failed to parse xml document"));
         goto cleanup;
     }
 
@@ -493,7 +511,7 @@ virNetworkDefPtr virNetworkDefParseFile(virConnectPtr conn,
     if (!xml) {
         if (conn && conn->err.code == VIR_ERR_NONE)
               virNetworkReportError(conn, VIR_ERR_XML_ERROR,
-                                    _("failed to parse xml document"));
+                                    "%s", _("failed to parse xml document"));
         goto cleanup;
     }
 
@@ -527,7 +545,7 @@ virNetworkDefPtr virNetworkDefParseNode(virConnectPtr conn,
 
     ctxt = xmlXPathNewContext(xml);
     if (ctxt == NULL) {
-        virNetworkReportError(conn, VIR_ERR_NO_MEMORY, NULL);
+        virReportOOMError(conn);
         goto cleanup;
     }
 
@@ -615,112 +633,102 @@ char *virNetworkDefFormat(virConnectPtr conn,
     return virBufferContentAndReset(&buf);
 
  no_memory:
-    virNetworkReportError(conn, VIR_ERR_NO_MEMORY, NULL);
+    virReportOOMError(conn);
     tmp = virBufferContentAndReset(&buf);
     VIR_FREE(tmp);
     return NULL;
 }
 
-int virNetworkSaveConfig(virConnectPtr conn,
-                         const char *configDir,
-                         const char *autostartDir,
-                         virNetworkObjPtr net)
+int virNetworkSaveXML(virConnectPtr conn,
+                      const char *configDir,
+                      virNetworkDefPtr def,
+                      const char *xml)
 {
-    char *xml;
+    char *configFile = NULL;
     int fd = -1, ret = -1;
     size_t towrite;
     int err;
 
-    if (!net->configFile &&
-        asprintf(&net->configFile, "%s/%s.xml",
-                 configDir, net->def->name) < 0) {
-        net->configFile = NULL;
-        virNetworkReportError(conn, VIR_ERR_NO_MEMORY, NULL);
-        goto cleanup;
-    }
-    if (!net->autostartLink &&
-        asprintf(&net->autostartLink, "%s/%s.xml",
-                 autostartDir, net->def->name) < 0) {
-        net->autostartLink = NULL;
-        virNetworkReportError(conn, VIR_ERR_NO_MEMORY, NULL);
-        goto cleanup;
-    }
-
-    if (!(xml = virNetworkDefFormat(conn,
-                                    net->newDef ? net->newDef : net->def)))
+    if ((configFile = virNetworkConfigFile(conn, configDir, def->name)) == NULL)
         goto cleanup;
 
     if ((err = virFileMakePath(configDir))) {
-        virNetworkReportError(conn, VIR_ERR_INTERNAL_ERROR,
-                              _("cannot create config directory %s: %s"),
-                              configDir, strerror(err));
+        virReportSystemError(conn, err,
+                             _("cannot create config directory '%s'"),
+                             configDir);
         goto cleanup;
     }
 
-    if ((err = virFileMakePath(autostartDir))) {
-        virNetworkReportError(conn, VIR_ERR_INTERNAL_ERROR,
-                              _("cannot create autostart directory %s: %s"),
-                              autostartDir, strerror(err));
-        goto cleanup;
-    }
-
-    if ((fd = open(net->configFile,
+    if ((fd = open(configFile,
                    O_WRONLY | O_CREAT | O_TRUNC,
                    S_IRUSR | S_IWUSR )) < 0) {
-        virNetworkReportError(conn, VIR_ERR_INTERNAL_ERROR,
-                              _("cannot create config file %s: %s"),
-                              net->configFile, strerror(errno));
+        virReportSystemError(conn, errno,
+                             _("cannot create config file '%s'"),
+                             configFile);
         goto cleanup;
     }
 
     towrite = strlen(xml);
     if (safewrite(fd, xml, towrite) < 0) {
-        virNetworkReportError(conn, VIR_ERR_INTERNAL_ERROR,
-                              _("cannot write config file %s: %s"),
-                              net->configFile, strerror(errno));
+        virReportSystemError(conn, errno,
+                             _("cannot write config file '%s'"),
+                             configFile);
         goto cleanup;
     }
 
     if (close(fd) < 0) {
-        virNetworkReportError(conn, VIR_ERR_INTERNAL_ERROR,
-                              _("cannot save config file %s: %s"),
-                              net->configFile, strerror(errno));
+        virReportSystemError(conn, errno,
+                             _("cannot save config file '%s'"),
+                             configFile);
         goto cleanup;
     }
 
     ret = 0;
 
  cleanup:
-    VIR_FREE(xml);
     if (fd != -1)
         close(fd);
 
+    VIR_FREE(configFile);
+
     return ret;
 }
 
+int virNetworkSaveConfig(virConnectPtr conn,
+                         const char *configDir,
+                         virNetworkDefPtr def)
+{
+    int ret = -1;
+    char *xml;
+
+    if (!(xml = virNetworkDefFormat(conn, def)))
+        goto cleanup;
+
+    if (virNetworkSaveXML(conn, configDir, def, xml))
+        goto cleanup;
+
+    ret = 0;
+cleanup:
+    VIR_FREE(xml);
+    return ret;
+}
+
+
 virNetworkObjPtr virNetworkLoadConfig(virConnectPtr conn,
-                                      virNetworkObjPtr *nets,
+                                      virNetworkObjListPtr nets,
                                       const char *configDir,
                                       const char *autostartDir,
-                                      const char *file)
+                                      const char *name)
 {
     char *configFile = NULL, *autostartLink = NULL;
     virNetworkDefPtr def = NULL;
     virNetworkObjPtr net;
     int autostart;
 
-    if (asprintf(&configFile, "%s/%s",
-                 configDir, file) < 0) {
-        configFile = NULL;
-        virNetworkReportError(conn, VIR_ERR_NO_MEMORY, NULL);
+    if ((configFile = virNetworkConfigFile(conn, configDir, name)) == NULL)
         goto error;
-    }
-    if (asprintf(&autostartLink, "%s/%s",
-                 autostartDir, file) < 0) {
-        autostartLink = NULL;
-        virNetworkReportError(conn, VIR_ERR_NO_MEMORY, NULL);
+    if ((autostartLink = virNetworkConfigFile(conn, autostartDir, name)) == NULL)
         goto error;
-    }
 
     if ((autostart = virFileLinkPointsTo(autostartLink, configFile)) < 0)
         goto error;
@@ -728,7 +736,7 @@ virNetworkObjPtr virNetworkLoadConfig(virConnectPtr conn,
     if (!(def = virNetworkDefParseFile(conn, configFile)))
         goto error;
 
-    if (!virFileMatchesNameSuffix(file, def->name, ".xml")) {
+    if (!STREQ(name, def->name)) {
         virNetworkReportError(conn, VIR_ERR_INTERNAL_ERROR,
                               _("Network config filename '%s'"
                                 " does not match network name '%s'"),
@@ -736,12 +744,20 @@ virNetworkObjPtr virNetworkLoadConfig(virConnectPtr conn,
         goto error;
     }
 
+    /* Generate a bridge if none is found, but don't check for collisions
+     * if a bridge is hardcoded, so the network is at least defined
+     */
+    if (!def->bridge && !(def->bridge = virNetworkAllocateBridge(conn, nets)))
+        goto error;
+
     if (!(net = virNetworkAssignDef(conn, nets, def)))
         goto error;
 
-    net->configFile = configFile;
-    net->autostartLink = autostartLink;
     net->autostart = autostart;
+    net->persistent = 1;
+
+    VIR_FREE(configFile);
+    VIR_FREE(autostartLink);
 
     return net;
 
@@ -753,7 +769,7 @@ error:
 }
 
 int virNetworkLoadAllConfigs(virConnectPtr conn,
-                             virNetworkObjPtr *nets,
+                             virNetworkObjListPtr nets,
                              const char *configDir,
                              const char *autostartDir)
 {
@@ -763,26 +779,30 @@ int virNetworkLoadAllConfigs(virConnectPtr conn,
     if (!(dir = opendir(configDir))) {
         if (errno == ENOENT)
             return 0;
-        virNetworkReportError(conn, VIR_ERR_INTERNAL_ERROR,
-                              _("Failed to open dir '%s': %s"),
-                              configDir, strerror(errno));
+        virReportSystemError(conn, errno,
+                             _("Failed to open dir '%s'"),
+                             configDir);
         return -1;
     }
 
     while ((entry = readdir(dir))) {
+        virNetworkObjPtr net;
+
         if (entry->d_name[0] == '.')
             continue;
 
-        if (!virFileHasSuffix(entry->d_name, ".xml"))
+        if (!virFileStripSuffix(entry->d_name, ".xml"))
             continue;
 
         /* NB: ignoring errors, so one malformed config doesn't
            kill the whole process */
-        virNetworkLoadConfig(conn,
-                             nets,
-                             configDir,
-                             autostartDir,
-                             entry->d_name);
+        net = virNetworkLoadConfig(conn,
+                                   nets,
+                                   configDir,
+                                   autostartDir,
+                                   entry->d_name);
+        if (net)
+            virNetworkObjUnlock(net);
     }
 
     closedir(dir);
@@ -791,23 +811,129 @@ int virNetworkLoadAllConfigs(virConnectPtr conn,
 }
 
 int virNetworkDeleteConfig(virConnectPtr conn,
+                           const char *configDir,
+                           const char *autostartDir,
                            virNetworkObjPtr net)
 {
-    if (!net->configFile || !net->autostartLink) {
-        virNetworkReportError(conn, VIR_ERR_INTERNAL_ERROR,
-                              _("no config file for %s"), net->def->name);
-        return -1;
-    }
+    char *configFile = NULL;
+    char *autostartLink = NULL;
+
+    if ((configFile = virNetworkConfigFile(conn, configDir, net->def->name)) == NULL)
+        goto error;
+    if ((autostartLink = virNetworkConfigFile(conn, autostartDir, net->def->name)) == NULL)
+        goto error;
 
     /* Not fatal if this doesn't work */
-    unlink(net->autostartLink);
+    unlink(autostartLink);
 
-    if (unlink(net->configFile) < 0) {
-        virNetworkReportError(conn, VIR_ERR_INTERNAL_ERROR,
-                              _("cannot remove config for %s: %s"),
-                              net->def->name, strerror(errno));
-        return -1;
+    if (unlink(configFile) < 0) {
+        virReportSystemError(conn, errno,
+                             _("cannot remove config file '%s'"),
+                             configFile);
+        goto error;
     }
 
     return 0;
+
+error:
+    VIR_FREE(configFile);
+    VIR_FREE(autostartLink);
+    return -1;
 }
+
+char *virNetworkConfigFile(virConnectPtr conn,
+                           const char *dir,
+                           const char *name)
+{
+    char *ret = NULL;
+
+    if (virAsprintf(&ret, "%s/%s.xml", dir, name) < 0) {
+        virReportOOMError(conn);
+        return NULL;
+    }
+
+    return ret;
+}
+
+int virNetworkBridgeInUse(const virNetworkObjListPtr nets,
+                          const char *bridge,
+                          const char *skipname)
+{
+    unsigned int i;
+    unsigned int ret = 0;
+
+    for (i = 0 ; i < nets->count ; i++) {
+        virNetworkObjLock(nets->objs[i]);
+        if (nets->objs[i]->def->bridge &&
+            STREQ(nets->objs[i]->def->bridge, bridge) &&
+            !(skipname && STREQ(nets->objs[i]->def->name, skipname)))
+                ret = 1;
+        virNetworkObjUnlock(nets->objs[i]);
+    }
+
+    return ret;
+}
+
+char *virNetworkAllocateBridge(virConnectPtr conn,
+                               const virNetworkObjListPtr nets)
+{
+
+    int id = 0;
+    char *newname;
+
+    do {
+        char try[50];
+
+        snprintf(try, sizeof(try), "virbr%d", id);
+
+        if (!virNetworkBridgeInUse(nets, try, NULL)) {
+            if (!(newname = strdup(try))) {
+                virReportOOMError(conn);
+                return NULL;
+            }
+            return newname;
+        }
+
+        id++;
+    } while (id <= MAX_BRIDGE_ID);
+
+    virNetworkReportError(conn, VIR_ERR_INTERNAL_ERROR,
+                          _("Bridge generation exceeded max id %d"),
+                          MAX_BRIDGE_ID);
+    return NULL;
+}
+
+int virNetworkSetBridgeName(virConnectPtr conn,
+                            const virNetworkObjListPtr nets,
+                            virNetworkDefPtr def) {
+
+    int ret = -1;
+
+    if (def->bridge) {
+        if (virNetworkBridgeInUse(nets, def->bridge, def->name)) {
+            networkReportError(conn, NULL, NULL, VIR_ERR_INTERNAL_ERROR,
+                               _("bridge name '%s' already in use."),
+                               def->bridge);
+            goto error;
+        }
+    } else {
+        /* Allocate a bridge name */
+        if (!(def->bridge = virNetworkAllocateBridge(conn, nets)))
+            goto error;
+    }
+
+    ret = 0;
+error:
+    return ret;
+}
+
+void virNetworkObjLock(virNetworkObjPtr obj)
+{
+    virMutexLock(&obj->lock);
+}
+
+void virNetworkObjUnlock(virNetworkObjPtr obj)
+{
+    virMutexUnlock(&obj->lock);
+}
+
