@@ -24,49 +24,9 @@
 #ifndef __VIR_STORAGE_BACKEND_H__
 #define __VIR_STORAGE_BACKEND_H__
 
-#include <libvirt/libvirt.h>
+#include "internal.h"
 #include "storage_conf.h"
 
-
-typedef const char *(*virStorageVolFormatToString)(virConnectPtr conn,
-                                                   int format);
-typedef int (*virStorageVolFormatFromString)(virConnectPtr conn,
-                                             const char *format);
-
-typedef const char *(*virStoragePoolFormatToString)(virConnectPtr conn,
-                                                    int format);
-typedef int (*virStoragePoolFormatFromString)(virConnectPtr conn,
-                                              const char *format);
-
-
-typedef struct _virStorageBackendVolOptions virStorageBackendVolOptions;
-typedef virStorageBackendVolOptions *virStorageBackendVolOptionsPtr;
-struct _virStorageBackendVolOptions {
-    virStorageVolFormatToString formatToString;
-    virStorageVolFormatFromString formatFromString;
-};
-
-
-/* Flags to indicate mandatory components in the pool source */
-enum {
-    VIR_STORAGE_BACKEND_POOL_SOURCE_HOST    = (1<<0),
-    VIR_STORAGE_BACKEND_POOL_SOURCE_DEVICE  = (1<<1),
-    VIR_STORAGE_BACKEND_POOL_SOURCE_DIR     = (1<<2),
-    VIR_STORAGE_BACKEND_POOL_SOURCE_ADAPTER = (1<<3),
-    VIR_STORAGE_BACKEND_POOL_SOURCE_NAME    = (1<<4),
-};
-
-typedef struct _virStorageBackendPoolOptions virStorageBackendPoolOptions;
-typedef virStorageBackendPoolOptions *virStorageBackendPoolOptionsPtr;
-struct _virStorageBackendPoolOptions {
-    int flags;
-    virStoragePoolFormatToString formatToString;
-    virStoragePoolFormatFromString formatFromString;
-};
-
-#define SOURCES_START_TAG "<sources>"
-#define SOURCES_END_TAG "</sources>"
-
 typedef char * (*virStorageBackendFindPoolSources)(virConnectPtr conn, const char *srcSpec, unsigned int flags);
 typedef int (*virStorageBackendStartPool)(virConnectPtr conn, virStoragePoolObjPtr pool);
 typedef int (*virStorageBackendBuildPool)(virConnectPtr conn, virStoragePoolObjPtr pool, unsigned int flags);
@@ -95,32 +55,30 @@ struct _virStorageBackend {
     virStorageBackendCreateVol createVol;
     virStorageBackendRefreshVol refreshVol;
     virStorageBackendDeleteVol deleteVol;
-
-    virStorageBackendPoolOptions poolOptions;
-    virStorageBackendVolOptions volOptions;
-
-    int volType;
 };
 
-
 virStorageBackendPtr virStorageBackendForType(int type);
-virStorageBackendPoolOptionsPtr virStorageBackendPoolOptionsForType(int type);
-virStorageBackendVolOptionsPtr virStorageBackendVolOptionsForType(int type);
-int virStorageBackendFromString(const char *type);
-const char *virStorageBackendToString(int type);
+
 
 int virStorageBackendUpdateVolInfo(virConnectPtr conn,
                                    virStorageVolDefPtr vol,
                                    int withCapacity);
 
-int virStorageBackendUpdateVolInfoFD(virConnectPtr conn,
-                                     virStorageVolDefPtr vol,
-                                     int fd,
-                                     int withCapacity);
+int virStorageBackendUpdateVolTargetInfo(virConnectPtr conn,
+                                         virStorageVolTargetPtr target,
+                                         unsigned long long *allocation,
+                                         unsigned long long *capacity);
+int virStorageBackendUpdateVolTargetInfoFD(virConnectPtr conn,
+                                           virStorageVolTargetPtr target,
+                                           int fd,
+                                           unsigned long long *allocation,
+                                           unsigned long long *capacity);
+
+void virStorageBackendWaitForDevices(virConnectPtr conn);
 
 char *virStorageBackendStablePath(virConnectPtr conn,
                                   virStoragePoolObjPtr pool,
-                                  char *devpath);
+                                  const char *devpath);
 
 typedef int (*virStorageBackendListVolRegexFunc)(virConnectPtr conn,
                                                  virStoragePoolObjPtr pool,
