@@ -8,14 +8,23 @@
 #include "testutils.h"
 #include "internal.h"
 #include "nodeinfo.h"
+#include "util.h"
+
+#ifndef __linux__
+
+static int
+mymain(int argc ATTRIBUTE_UNUSED, char **argv ATTRIBUTE_UNUSED)
+{
+    exit (77); /* means 'test skipped' for automake */
+}
+
+#else
 
 static char *progname;
 static char *abs_srcdir;
 
 #define MAX_FILE 4096
 
-#ifdef __linux__
-
 extern int linuxNodeInfoCPUPopulate(virConnectPtr conn, FILE *cpuinfo, virNodeInfoPtr nodeinfo);
 
 static int linuxTestCompareFiles(const char *cpuinfofile, const char *outputfile) {
@@ -63,14 +72,12 @@ static int linuxTestNodeInfo(const void *data) {
              abs_srcdir, (const char*)data);
     return linuxTestCompareFiles(cpuinfo, output);
 }
-#endif
 
 
 static int
 mymain(int argc, char **argv)
 {
     int ret = 0;
-#ifdef __linux__
     int i;
     const char *nodeData[] = {
         "nodeinfo-1",
@@ -95,12 +102,14 @@ mymain(int argc, char **argv)
 
     virInitialize();
 
-    for (i = 0 ; i < (sizeof(nodeData)/sizeof(nodeData[0])) ; i++)
+    for (i = 0 ; i < ARRAY_CARDINALITY(nodeData); i++)
       if (virtTestRun(nodeData[i], 1, linuxTestNodeInfo, nodeData[i]) != 0)
         ret = -1;
-#endif
 
     return(ret==0 ? EXIT_SUCCESS : EXIT_FAILURE);
 }
 
+#endif /* __linux__ */
+
 VIRT_TEST_MAIN(mymain)
+
