@@ -1,6 +1,5 @@
 #include <config.h>
 
-#ifdef WITH_XEN
 #include <sys/utsname.h>
 #include <stdlib.h>
 
@@ -52,4 +51,3 @@ cleanup:
     virCapabilitiesFree(caps);
     return NULL;
 }
-#endif
