@@ -8,8 +8,6 @@
 #include <fcntl.h>
 #include <unistd.h>
 
-#if WITH_XEN
-
 #include "internal.h"
 #include "xend_internal.h"
 #include "testutils.h"
@@ -37,7 +35,8 @@ static int testCompareFiles(const char *xml, const char *sexpr,
   if (virtTestLoadFile(sexpr, &sexprPtr, MAX_FILE) < 0)
       goto fail;
 
-  if (!(def = virDomainDefParseString(NULL, caps, xmlData)))
+  if (!(def = virDomainDefParseString(NULL, caps, xmlData,
+                                      VIR_DOMAIN_XML_INACTIVE)))
       goto fail;
 
   if (!(gotsexpr = xenDaemonFormatSxpr(NULL, def, xendConfigVersion)))
@@ -127,6 +126,7 @@ mymain(int argc, char **argv)
     DO_TEST("net-routed", "net-routed", "pvtest", 2);
     DO_TEST("net-bridged", "net-bridged", "pvtest", 2);
     DO_TEST("net-e1000", "net-e1000", "pvtest", 2);
+    DO_TEST("bridge-ipaddr", "bridge-ipaddr", "pvtest", 2);
     DO_TEST("no-source-cdrom", "no-source-cdrom", "test", 2);
 
     DO_TEST("fv-utc", "fv-utc", "fvtest", 1);
@@ -155,8 +155,3 @@ mymain(int argc, char **argv)
 
 VIRT_TEST_MAIN(mymain)
 
-#else /* WITH_XEN */
-
-int main (void) { exit (77); /* means 'test skipped' for automake */ }
-
-#endif /* ! WITH_XEN */
